#include "timer.h"

/**
 * Poll, if timer has elapsed once again.
 *
 * Use only in polling mode!
 *
 * @return	true if timer has elapsed at least once since last call to this method
 * 			false otherwise
 */
bool
TIMER_pollHasElapsed() {

	if (CCU40_CC41->INTS & (1<<CCU4_CC4_INTS_PMUS_Pos)) {
		/* clears the period match interrupt, if ISR is not called, otherwise the flag will be kept set */
		CCU40_CC41->SWR		|= 1U << CCU4_CC4_SWR_RPM_Pos;
		return true;
	}
	return false;
}

/**
 * Configures the timer to call ISR routine.
 *
 * @param[in]	freqDivider		base frequency divider as power of two [0..15]
 * @param[in]	periodCycles	amount of cycles for a full period [1..65536]
 * @param[in]	useIrq			if true, period match results in the ISR being called
 */
void
TIMER_init(
		uint32_t		freqDivider,
		uint32_t		periodCycles,
		bool			useIrq) {

	/* enable the CCU4 module and pull it from reset */
	SCU_CLK->CLKSET		= 1U << SCU_CLK_CLKSET_CCUCEN_Pos;		/* enable CCU clock (fCCU; for both CCU4 and CCU8 modules) */
	SCU_CLK->CGATCLR0	= 1U << SCU_CLK_CGATSTAT0_CCU40_Pos;	/* disable clock gating of CCU40 timer */
	SCU_RESET->PRCLR0	= 1U << SCU_RESET_PRSTAT0_CCU40RS_Pos;	/* de-assert peripheral reset of CCU40 */

	CCU40->GIDLC		|= 1U << CCU4_GIDLC_SPRB_Pos			/* start pre-scaler */
						|  1U << CCU4_GIDLC_CS1I_Pos;			/* enable the clock for slice 1 in CCU43 by removing it from idle mode */

	/* configure the timer slice */
	CCU40_CC41->TC		= 0U << CCU4_CC4_TC_TCM_Pos			/* edge-aligned mode */
						| 0U << CCU4_CC4_TC_TSSM_Pos		/* disabled single-shot mode (=repeat mode) */
						| 0U << CCU4_CC4_TC_CLST_Pos		/* do not perform shadow transfer when timer is cleared */
						| 0U << CCU4_CC4_TC_DITHE_Pos		/* dithering is neither applied to period value nor to compare value */
						| 0U << CCU4_CC4_TC_FPE_Pos			/* floating prescaler mode disabled */
						| 0U << CCU4_CC4_TC_MCME_Pos;		/* multi-channel mode disabled */

	CCU40_CC41->CMC 	= 0U << CCU4_CC4_CMC_TCE_Pos;		/* do not concatenate this timer with previous slice */
	CCU40_CC41->PSC		= freqDivider & 0x0F;				/* pre-scaler divides module clock of 144 MHz down by 2^8 to 562500 Hz */
	CCU40_CC41->DITS	= 0U;								/* dither compare value 0 (not used) */
	CCU40_CC41->PSL		= 0;								/* passive level of output pin = low */
	CCU40_CC41->FPCS	= 0;								/* do not use floating prescaler mode */

	/* programs the timer period and compare register according and do the shadow transfer */
	CCU40_CC41->PRS		= (periodCycles - 1) & 0xFFFF;		/* period [cycles] minus 1 (shadow register) */
	CCU40_CC41->CRS		= 0;								/* 100 % duty cycle, because timer count will always be greater/equal compare value */
	CCU40->GCSS			|= 1U << CCU4_GCSS_S1SE_Pos			/* enable shadow transfer of both period and compare values */
						| 1U << CCU4_GCSS_S1PSE_Pos;

	/* binds a period match event to an NVIC node (service request line 1) */
	CCU40_CC41->SRS		|= 1U << CCU4_CC4_SRS_POSR_Pos;		/* forward period match to Service Request Line 1 of the slice */
	CCU40_CC41->INTE	|= 1U << CCU4_CC4_INTE_PME_Pos;		/* enables interrupt pulse generation every time a period match occurs */
	CCU40_CC41->TCCLR	= 1U << CCU4_CC4_TCCLR_TCC_Pos;		/* clears the timer register (=count) */

	if (useIrq) {
		NVIC->IP[CCU40_1_IRQn]	= 32U << 2;						/* set middle priority for interrupt (lowest two bits are 0) */
		NVIC->ISER[1]			= 1U << (CCU40_1_IRQn & 0x1F);	/* enable interrupt in NVIC (1 bit per interrupt; 32 interrupts per register; 4-element array for a total of 128 interrupts) */
	}
}
