/**
 * 16-bit continuously running timer which can either call ISR when it elapses or can be polled by user.
 *
 * MODES:
 * By parameter "useIrq" in initialization routine, either the interrupt-driven mode (true) or the polling mode (false) is chosen.
 * In polling mode, user code needs to call TIMER_pollHasElapsed() regularly to check for elapsed timer.
 * In interrupt-driven mode, the user needs to implement an ISR callback. No additional code in the ISR is required:
 *
 * void CCU40_1_IRQHandler() {
 * 		// put user-code here
 * }
 *
 * ELAPSED TIME / FREQUENCY:
 * The frequency divider divides the fCCU clock of 144 MHz into a smaller base clock, which is fed into the 16-bit benchmark timer.
 * A high divider results in small timer frequency, which allows less fine-grained, but longer time spans.
 * A small divider results in high timer frequency, which allows fine-grained, but short time spans.
 * The divider is defined by "freqDivider" [0;15] with divider = 2^dividerPotency. This results in the following possible benchmark timer values.
 *
 * freqDivider|timer frequency|cycle time| max. timespan
 * -----------+---------------+----------+---------------
 *      0		   144 MHz	     ~6.94 ns     ~0.45 ms
 *      1			72 MHz	    ~13.89 ns     ~0.91 ms
 *      2			36 MHz	    ~27.78 ns	  ~1.82 ms
 *      3		    18 MHz	    ~55.56 ns	  ~3.64 ms
 *      4			 9 MHz	   ~111.11 ns	  ~7.28 ms
 *      5		   4.5 MHz	   ~222.22 ns    ~14.56 ms
 *      6		  ~2.3 MHz	   ~444.44 ns    ~29.13 ms
 *      7		  ~1.1 MHz	   ~888.88 ns    ~58.25 ms
 *      8		 562.5 kHz	     ~1.78 us     ~0.12 s
 *      9		~281.3 kHz		 ~3.56 us	  ~0.23 s
 *     10		~140.6 kHz		 ~7.11 us	  ~0.47 s
 *     11		 ~70.3 kHz	    ~14.22 us	  ~0.93 s
 *     12		 ~35.2 kHz	    ~28.44 us	  ~1.86 s
 *     13		 ~17.6 kHz	    ~56.89 us	  ~3.73 s
 *     14		  ~8.8 kHz	   ~113.78 us     ~7.45 s
 *     15		  ~4.4 kHz 	   ~227.56 us	 ~14.91 s
 *
 * Typical values for timers:
 * 1-second-timer .. TIMER_init(12, 35156, ...)
 *   100-ms-timer .. TIMER_init( 8, 56250, ...)
 *    10-ms-timer .. TIMER_init( 5, 45000, ...)
 *     1-ms-timer .. TIMER_init( 2, 36000, ...)
 *
 * DEPENDENCIES / ALLOCATIONS:
 * Uses CCU4 module, instance 0, slice 1 (CCU40_CC41).
 * Uses interrupt CCU40_1_IRQn to signal elapsed timer.
 */
#ifndef _FOCUS_TIMER_H
#define _FOCUS_TIMER_H

#include <xmc_common.h>

#include <stdint.h>

/** Starts the timer. */
static inline void
TIMER_start() {

	CCU40_CC41->TCCLR = 1U << CCU4_CC4_TCCLR_TCC_Pos;	/* clear timer count */
	CCU40_CC41->TCSET = 1U << CCU4_CC4_TCSET_TRBS_Pos;
}

/** Stops the timer. */
static inline void
TIMER_stop() {

	CCU40_CC41->TCCLR = 1U << CCU4_CC4_TCCLR_TRBC_Pos;
}

/** Poll, if timer has elapsed once again. */
extern bool
TIMER_pollHasElapsed();

/** Initialize timer. */
extern void
TIMER_init(
		uint32_t		freqDivider,
		uint32_t		periodCycles,
		bool			useIrq);

#endif
